@@ -1,6 +1,6 @@
 # -*-python-*-
 #
-# Copyright (C) 1999-2010 The ViewCVS Group. All Rights Reserved.
+# Copyright (C) 1999-2012 The ViewCVS Group. All Rights Reserved.
 #
 # By using this file, you agree to the terms and conditions set forth in
 # the LICENSE.html file which can be found at the top level of the ViewVC
@@ -88,6 +88,11 @@ import fnmatch
 #                                           |  vhosts   |
 #                                           |           |
 #                                           `-----------'
+#                                           ,-----------.
+#                                           |           |
+#                                           |   query   |
+#                                           |           |
+#                                           `-----------'
 #
 # ### TODO:  Figure out what this all means for the 'kv' stuff.
 #
@@ -100,12 +105,14 @@ class Config:
     'cvsdb',
     'general',
     'options',
+    'query',
     'templates',
     'utilities',
     )
   _force_multi_value = (
     # Configuration values with multiple, comma-separated values.
     'allowed_views',
+    'custom_log_formatting',
     'cvs_roots',
     'kv_files',
     'languages',
@@ -145,11 +152,11 @@ class Config:
     self.conf_path = os.path.isfile(pathname) and pathname or None
     self.base = os.path.dirname(pathname)
     self.parser = ConfigParser.ConfigParser()
+    self.parser.optionxform = lambda x: x # don't case-normalize option names.
     self.parser.read(self.conf_path or [])
     
     for section in self.parser.sections():
-      if self._is_allowed_section(self.parser, section,
-                                  self._base_sections):
+      if self._is_allowed_section(section, self._base_sections):
         self._process_section(self.parser, section, section)
 
     if vhost and self.parser.has_section('vhosts'):
@@ -172,6 +179,7 @@ class Config:
       fname = string.replace(fname, '%lang%', language)
 
       parser = ConfigParser.ConfigParser()
+      parser.optionxform = lambda x: x # don't case-normalize option names.
       parser.read(os.path.join(self.base, fname))
       for section in parser.sections():
         for option in parser.options(section):
@@ -214,7 +222,7 @@ class Config:
 
       setattr(sc, opt, value)
 
-  def _is_allowed_section(self, parser, section, allowed_sections):
+  def _is_allowed_section(self, section, allowed_sections):
     """Return 1 iff SECTION is an allowed section, defined as being
     explicitly present in the ALLOWED_SECTIONS list or present in the
     form 'someprefix-*' in that list."""
@@ -227,7 +235,7 @@ class Config:
         return 1
     return 0
 
-  def _is_allowed_override(self, parser, sectype, secspec, section):
+  def _is_allowed_override(self, sectype, secspec, section):
     """Test if SECTION is an allowed override section for sections of
     type SECTYPE ('vhosts' or 'root', currently) and type-specifier
     SECSPEC (a rootname or vhostname, currently).  If it is, return
@@ -240,7 +248,7 @@ class Config:
     if section[:lcv] != cv:
       return None
     base_section = section[lcv:]
-    if self._is_allowed_section(parser, base_section,
+    if self._is_allowed_section(base_section,
                                 self._allowed_overrides[sectype]):
       return base_section
     raise IllegalOverrideSection(sectype, section)
@@ -253,8 +261,7 @@ class Config:
 
     # Overlay any option sections associated with this vhost name.
     for section in parser.sections():
-      base_section = self._is_allowed_override(parser, 'vhost',
-                                               canon_vhost, section)
+      base_section = self._is_allowed_override('vhost', canon_vhost, section)
       if base_section:
         self._process_section(parser, section, base_section)
 
@@ -280,8 +287,7 @@ class Config:
       return
 
     for section in self.parser.sections():
-      base_section = self._is_allowed_override(self.parser, 'root',
-                                               rootname, section)
+      base_section = self._is_allowed_override('root', rootname, section)
       if base_section:
         # We can currently only deal with root overlays happening
         # once, so check that we've not yet done any overlaying of
@@ -322,7 +328,7 @@ class Config:
     assert(self.root_options_overlayed == 0)
 
     if not self.conf_path:
-      return None
+      return None, {}
 
     # Figure out the authorizer by searching first for a per-root
     # override, then falling back to the base/vhost configuration.
@@ -393,6 +399,7 @@ class Config:
     self.options.allowed_views = ['annotate', 'diff', 'markup', 'roots']
     self.options.authorizer = None
     self.options.mangle_email_addresses = 0
+    self.options.custom_log_formatting = []
     self.options.default_file_view = "log"
     self.options.http_expiration_time = 600
     self.options.generate_etags = 1
@@ -415,6 +422,7 @@ class Config:
     self.options.template_dir = "templates"
     self.options.docroot = None
     self.options.show_subdir_lastmod = 0
+    self.options.show_roots_lastmod = 0
     self.options.show_logs = 1
     self.options.show_log_in_markup = 1
     self.options.cross_copies = 1
@@ -428,6 +436,7 @@ class Config:
     self.options.use_re_search = 0
     self.options.dir_pagesize = 0
     self.options.log_pagesize = 0
+    self.options.log_pagesextra = 3
     self.options.limit_changes = 100
 
     self.templates.diff = None
@@ -453,6 +462,8 @@ class Config:
     self.cvsdb.rss_row_limit = 100
     self.cvsdb.check_database_for_root = 0
 
+    self.query.viewvc_base_url = None
+    
 def _startswith(somestr, substr):
   return somestr[:len(substr)] == substr
 
