@@ -1,6 +1,6 @@
 # -*-python-*-
 #
-# Copyright (C) 1999-2010 The ViewCVS Group. All Rights Reserved.
+# Copyright (C) 1999-2012 The ViewCVS Group. All Rights Reserved.
 #
 # By using this file, you agree to the terms and conditions set forth in
 # the LICENSE.html file which can be found at the top level of the ViewVC
@@ -186,7 +186,7 @@ class CgiServer(Server):
   def FieldStorage(fp=None, headers=None, outerboundary="",
                  environ=os.environ, keep_blank_values=0, strict_parsing=0):
     return cgi.FieldStorage(fp, headers, outerboundary, environ,
-      keep_blank_values, strict_parsing)
+                            keep_blank_values, strict_parsing)
 
   def write(self, s):
     sys.stdout.write(s)
@@ -233,9 +233,6 @@ class WsgiServer(Server):
     self.header(status='301 Moved')
     self._wsgi_write('This document is located <a href="%s">here</a>.' % url)
 
-  def escape(self, s, quote = None):
-    return cgi.escape(s, quote)
-
   def getenv(self, name, value=None):
     return self._environ.get(name, value)
 
@@ -244,7 +241,7 @@ class WsgiServer(Server):
 
   def FieldStorage(self, fp=None, headers=None, outerboundary="",
                    environ=os.environ, keep_blank_values=0, strict_parsing=0):
-    return cgi.FieldStorage(self._environ["wsgi.input"], self._headers,
+    return cgi.FieldStorage(self._environ["wsgi.input"], headers,
                             outerboundary, self._environ, keep_blank_values,
                             strict_parsing)
 
